import React, {FC, useEffect, useState} from 'react'
import MarkdownRenderer from './MarkdownRenderer';
import {socket} from '../../../../utils/components/socket';
import {CopyToClipboard} from '../../../../utils/components/CopyToClipboard';
import DynamicForm from './DynamicForm';

const GptSandboxArea: FC = () => {

  const jsonData = {
    "name": "John Doe",
    "age": 30,
    "Occupation": "Software Engineer",
    "email": "johndoe@example.com",
    "isSubscribed": true,
    "preferences": {
      "newsletter": true,
      "sms": false
    }
  };

  interface GenerationStatus {
    updateTimestamp: Date;
    gptGenerationRequestId?: string;
    message: string;
    numberOfPrompts?: number;
    completedPrompts?: number;
  }

  const [selectedFile, setSelectedFile] = useState<File | null>(null);
  const [aiResponse, setAiResponse] = useState<string>('');
  const [aiLoadingText, setAiLoadingText] = useState<string>('');

  const [generationStatus, setGenerationStatus] = useState<GenerationStatus>();


  const handleFileSelect = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files ? event.target.files[0] : null;
    socket.emit('leave_all_rooms');
    setSelectedFile(file);
    setAiResponse('');
    setAiLoadingText('');
    setGenerationStatus({
      updateTimestamp: new Date(),
      gptGenerationRequestId: "",
      message: "Prompt file selected. Ready to submit.",
      numberOfPrompts: -1,
      completedPrompts: -1
    });
  };

  const pollForStatus = () => {
    // Make a call to get the status of the generation process
    fetch('http://localhost:8080/api/klokgpt/generate/v1/' + generationStatus?.gptGenerationRequestId )
      .then(response => response.json())
      .then(data => {
        setGenerationStatus({
          updateTimestamp: new Date(),
          gptGenerationRequestId: data.id,
          message: data.message,
          numberOfPrompts: data.numberOfPrompts,
          completedPrompts: data.promptResponses.length
        });

        console.log('Response from server:', data);
      })
      .catch(error => {
        console.error('Error:', error)
        setGenerationStatus({
          updateTimestamp: new Date(),
          message: error.message
        });
      });
  };

  const handleSubmit = () => {
    if (selectedFile) {
      const reader = new FileReader();

      reader.onload = function(event) {
        const fileContent = event.target?.result as string;

        // Construct the JSON structure
        const klokGptGenerationRequest = {
          prompt: fileContent,
          typeOfGeneration: "new_offramp",
        };

        console.log("Data to send:", klokGptGenerationRequest);

        // Sending a POST request to the server
        fetch('http://localhost:8080/api/klokgpt/generate/v1', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(klokGptGenerationRequest),
        })
          .then(response => response.json())
          .then(data => {
            console.log('Response from server:', data);


            // Update the responses in the UI
            setGenerationStatus({
              updateTimestamp: new Date(),
              gptGenerationRequestId: data.id,
              message: data.message,
              numberOfPrompts: data.numberOfPrompts,
              completedPrompts: data.promptResponses.length
            });
            setAiResponse(''); // Clear the previous response (if any)
            setAiLoadingText('Generation process starting...');

            socket.emit('leave_all_rooms');
            socket.emit('join_room', data.id);
          })
          .catch(error => {
            console.error('Error:', error);
            setAiResponse('An error occurred during the generation process.');
          });
      };

      reader.readAsText(selectedFile); // Read the file content as text

      // Update UI to indicate that the process is starting
      setAiResponse('');
      setGenerationStatus({
        updateTimestamp: new Date(),
        gptGenerationRequestId: "",
        message: "Calling the server... Initiating the generation process...",
        numberOfPrompts: -1,
        completedPrompts: -1
      });
    } else {
      alert('Please select a file first.');
    }
  };

  // Use useEffect to listen to socket messages
  useEffect(() => {
    socket.on('AIResponseToken', (message: string) => {
      console.log('Received message:', message);
      setAiResponse((prevResponse) => prevResponse + message); // Update aiResponse with new token
      setAiLoadingText('');
    });

    return () => {
      socket.off('AIResponseToken'); // Cleanup to avoid memory leaks
    };
  }, []);

  useEffect(() => {
    // Check if the loading text includes 'Generation process starting'
    if (aiLoadingText.includes('Generation process starting')) {
      let dotCount = (aiLoadingText.match(/\./g) || []).length; // Count the current number of dots
      dotCount = dotCount === 5 ? 1 : dotCount + 1; // Reset after 9 dots or increment

      const timeoutId = setTimeout(() => {
        setAiLoadingText(`Generation process starting${'.'.repeat(dotCount)}`);
      }, 500); // Delay for 500ms

      // Cleanup function to clear the timeout if aiLoadingText changes or component unmounts
      return () => clearTimeout(timeoutId);
    }
  }, [aiLoadingText]);

  useEffect(() => {
    // Check if the loading text includes 'Generation process starting'
    if (generationStatus?.gptGenerationRequestId && generationStatus.gptGenerationRequestId.length > 0 && generationStatus?.completedPrompts !== generationStatus?.numberOfPrompts) {
      const timeoutId = setTimeout(() => {
        pollForStatus()
      }, 2000); // Delay for 500ms

      // Cleanup function to clear the timeout if aiLoadingText changes or component unmounts
      return () => clearInterval(timeoutId);
    }
  }, [generationStatus]);

  return (
    <div>
      <div className="row mb-5">
        <div className="col-8 mb-5">
          <div className="card mb-4">

            <div className="card-body">
              <div className="form-group">
                <br />
                <h3>Upload a prompt file</h3>
                <input
                  type="file"
                  id="fileInput"
                  className="form-control"
                  onChange={handleFileSelect}
                />
              </div>
              <button
                id="sendRequest"
                className="btn btn-primary mt-3"
                onClick={handleSubmit}
                disabled={!selectedFile}
              >
                Submit
              </button>
              <br />
              <br />
              <br />

            </div>
          </div>
        </div>
        <div className="col-4">
          <div className="card">
            <div className="card-body">
              <br />
              <h3>Status</h3>
              {generationStatus && (
                  <div>
                    <p>Updated: {generationStatus.updateTimestamp.toISOString()}</p>
                    <p>Request ID: {generationStatus.gptGenerationRequestId}</p>
                    <p>Message: {generationStatus.message}</p>
                    <p>Number of prompts: {generationStatus.numberOfPrompts}</p>
                    <p>Completed prompts: {generationStatus.completedPrompts}</p>
                  </div>
                )}
            </div>
          </div>
        </div>
      </div>
      <div className="row">
        <div className="col-12">
          <div className="card">
            <div className="card-body">
              <h2>{aiLoadingText}</h2>
              <MarkdownRenderer content={aiResponse} />
              {generationStatus &&
                generationStatus.completedPrompts === generationStatus.numberOfPrompts &&
                generationStatus.numberOfPrompts && generationStatus.numberOfPrompts > 0 && (
                  <div>
                    <CopyToClipboard textToCopy={aiResponse} />
                    <a
                      href={`http://localhost:8080/api/klokgpt/download/v1/${generationStatus.gptGenerationRequestId}`}
                      className="btn btn-primary mt-3"
                    >
                      Download as Zip
                    </a>
                  </div>
                )}
            </div>
          </div>
        </div>
      </div>
    </div>
  )
}

export {GptSandboxArea}